@@ -38,7 +38,10 @@
 #include <stdlib.h>
 #include <unistd.h>
 
+#include <rdr/Exception.h>
+
 #include <network/TcpSocket.h>
+
 #include <rfb/LogWriter.h>
 #include <rfb/Configuration.h>
 #include <rfb/util.h>
@@ -82,15 +85,15 @@ int network::findFreeTcpPort (void)
   addr.sin_addr.s_addr = INADDR_ANY;
 
   if ((sock = socket (AF_INET, SOCK_STREAM, 0)) < 0)
-    throw SocketException ("unable to create socket", errorNumber);
+    throw socket_error("Unable to create socket", errorNumber);
 
   addr.sin_port = 0;
   if (bind (sock, (struct sockaddr *)&addr, sizeof (addr)) < 0)
-    throw SocketException ("unable to find free port", errorNumber);
+    throw socket_error("Unable to find free port", errorNumber);
 
   socklen_t n = sizeof(addr);
   if (getsockname (sock, (struct sockaddr *)&addr, &n) < 0)
-    throw SocketException ("unable to get port number", errorNumber);
+    throw socket_error("Unable to get port number", errorNumber);
 
   closesocket (sock);
   return ntohs(addr.sin_port);
@@ -129,17 +132,17 @@ TcpSocket::TcpSocket(const char *host, int port)
   memset(&hints, 0, sizeof(struct addrinfo));
   hints.ai_family = AF_UNSPEC;
   hints.ai_socktype = SOCK_STREAM;
-  hints.ai_canonname = NULL;
-  hints.ai_addr = NULL;
-  hints.ai_next = NULL;
+  hints.ai_canonname = nullptr;
+  hints.ai_addr = nullptr;
+  hints.ai_next = nullptr;
 
-  if ((result = getaddrinfo(host, NULL, &hints, &ai)) != 0) {
-    throw GAIException("unable to resolve host by name", result);
+  if ((result = getaddrinfo(host, nullptr, &hints, &ai)) != 0) {
+    throw getaddrinfo_error("Unable to resolve host by name", result);
   }
 
   sock = -1;
   err = 0;
-  for (current = ai; current != NULL; current = current->ai_next) {
+  for (current = ai; current != nullptr; current = current->ai_next) {
     int family;
     vnc_sockaddr_t sa;
     socklen_t salen;
@@ -168,14 +171,14 @@ TcpSocket::TcpSocket(const char *host, int port)
     else
       sa.u.sin6.sin6_port = htons(port);
 
-    getnameinfo(&sa.u.sa, salen, ntop, sizeof(ntop), NULL, 0, NI_NUMERICHOST);
+    getnameinfo(&sa.u.sa, salen, ntop, sizeof(ntop), nullptr, 0, NI_NUMERICHOST);
     vlog.debug("Connecting to %s [%s] port %d", host, ntop, port);
 
     sock = socket (family, SOCK_STREAM, 0);
     if (sock == -1) {
       err = errorNumber;
       freeaddrinfo(ai);
-      throw SocketException("unable to create socket", err);
+      throw socket_error("Unable to create socket", err);
     }
 
   /* Attempt to connect to the remote host */
@@ -200,9 +203,9 @@ TcpSocket::TcpSocket(const char *host, int port)
 
   if (sock == -1) {
     if (err == 0)
-      throw Exception("No useful address for host");
+      throw std::runtime_error("No useful address for host");
     else
-      throw SocketException("unable to connect to socket", err);
+      throw socket_error("Unable to connect to socket", err);
   }
 
   // Take proper ownership of the socket
@@ -217,7 +220,7 @@ const char* TcpSocket::getPeerAddress() {
   socklen_t sa_size = sizeof(sa);
 
   if (getpeername(getFd(), &sa.u.sa, &sa_size) != 0) {
-    vlog.error("unable to get peer name for socket");
+    vlog.error("Unable to get peer name for socket");
     return "(N/A)";
   }
 
@@ -228,10 +231,10 @@ const char* TcpSocket::getPeerAddress() {
     buffer[0] = '[';
 
     ret = getnameinfo(&sa.u.sa, sizeof(sa.u.sin6),
-                      buffer + 1, sizeof(buffer) - 2, NULL, 0,
+                      buffer + 1, sizeof(buffer) - 2, nullptr, 0,
                       NI_NUMERICHOST);
     if (ret != 0) {
-      vlog.error("unable to convert peer name to a string");
+      vlog.error("Unable to convert peer name to a string");
       return "(N/A)";
     }
 
@@ -244,15 +247,15 @@ const char* TcpSocket::getPeerAddress() {
     char *name;
 
     name = inet_ntoa(sa.u.sin.sin_addr);
-    if (name == NULL) {
-      vlog.error("unable to convert peer name to a string");
+    if (name == nullptr) {
+      vlog.error("Unable to convert peer name to a string");
       return "(N/A)";
     }
 
     return name;
   }
 
-  vlog.error("unknown address family for socket");
+  vlog.error("Unknown address family for socket");
   return "";
 }
 
@@ -281,7 +284,7 @@ bool TcpSocket::enableNagles(bool enable) {
   if (setsockopt(getFd(), IPPROTO_TCP, TCP_NODELAY,
                  (char *)&one, sizeof(one)) < 0) {
     int e = errorNumber;
-    vlog.error("unable to setsockopt TCP_NODELAY: %d", e);
+    vlog.error("Unable to setsockopt TCP_NODELAY: %d", e);
     return false;
   }
   return true;
@@ -299,7 +302,7 @@ TcpListener::TcpListener(const struct sockaddr *listenaddr,
   int sock;
 
   if ((sock = socket (listenaddr->sa_family, SOCK_STREAM, 0)) < 0)
-    throw SocketException("unable to create listening socket", errorNumber);
+    throw socket_error("Unable to create listening socket", errorNumber);
 
   memcpy (&sa, listenaddr, listenaddrlen);
 #ifdef IPV6_V6ONLY
@@ -307,7 +310,7 @@ TcpListener::TcpListener(const struct sockaddr *listenaddr,
     if (setsockopt (sock, IPPROTO_IPV6, IPV6_V6ONLY, (char*)&one, sizeof(one))) {
       int e = errorNumber;
       closesocket(sock);
-      throw SocketException("unable to set IPV6_V6ONLY", e);
+      throw socket_error("Unable to set IPV6_V6ONLY", e);
     }
   }
 #endif /* defined(IPV6_V6ONLY) */
@@ -325,21 +328,21 @@ TcpListener::TcpListener(const struct sockaddr *listenaddr,
                  (char *)&one, sizeof(one)) < 0) {
     int e = errorNumber;
     closesocket(sock);
-    throw SocketException("unable to create listening socket", e);
+    throw socket_error("Unable to create listening socket", e);
   }
 #endif
 
   if (bind(sock, &sa.u.sa, listenaddrlen) == -1) {
     int e = errorNumber;
     closesocket(sock);
-    throw SocketException("failed to bind socket", e);
+    throw socket_error("Failed to bind socket", e);
   }
 
   listen(sock);
 }
 
-Socket* TcpListener::createSocket(int fd) {
-  return new TcpSocket(fd);
+Socket* TcpListener::createSocket(int fd_) {
+  return new TcpSocket(fd_);
 }
 
 std::list<std::string> TcpListener::getMyAddresses() {
@@ -352,15 +355,15 @@ std::list<std::string> TcpListener::getMyAddresses() {
   hints.ai_flags = AI_PASSIVE | AI_NUMERICSERV;
   hints.ai_family = AF_UNSPEC;
   hints.ai_socktype = SOCK_STREAM;
-  hints.ai_canonname = NULL;
-  hints.ai_addr = NULL;
-  hints.ai_next = NULL;
+  hints.ai_canonname = nullptr;
+  hints.ai_addr = nullptr;
+  hints.ai_next = nullptr;
 
   // Windows doesn't like NULL for service, so specify something
-  if ((getaddrinfo(NULL, "1", &hints, &ai)) != 0)
+  if ((getaddrinfo(nullptr, "1", &hints, &ai)) != 0)
     return result;
 
-  for (current= ai; current != NULL; current = current->ai_next) {
+  for (current= ai; current != nullptr; current = current->ai_next) {
     char addr[INET6_ADDRSTRLEN];
 
     switch (current->ai_family) {
@@ -377,7 +380,7 @@ std::list<std::string> TcpListener::getMyAddresses() {
     }
 
     getnameinfo(current->ai_addr, current->ai_addrlen, addr, INET6_ADDRSTRLEN,
-                NULL, 0, NI_NUMERICHOST);
+                nullptr, 0, NI_NUMERICHOST);
 
     result.push_back(addr);
   }
@@ -417,7 +420,7 @@ void network::createLocalTcpListeners(std::list<SocketListener*> *listeners,
   ai[1].ai_family = sa[1].u.sin6.sin6_family;
   ai[1].ai_addr = &sa[1].u.sa;
   ai[1].ai_addrlen = sizeof(sa[1].u.sin6);
-  ai[1].ai_next = NULL;
+  ai[1].ai_next = nullptr;
 
   createTcpListeners(listeners, ai);
 }
@@ -436,14 +439,14 @@ void network::createTcpListeners(std::list<SocketListener*> *listeners,
   hints.ai_flags = AI_PASSIVE | AI_NUMERICSERV;
   hints.ai_family = AF_UNSPEC;
   hints.ai_socktype = SOCK_STREAM;
-  hints.ai_canonname = NULL;
-  hints.ai_addr = NULL;
-  hints.ai_next = NULL;
+  hints.ai_canonname = nullptr;
+  hints.ai_addr = nullptr;
+  hints.ai_next = nullptr;
 
   snprintf (service, sizeof (service) - 1, "%d", port);
   service[sizeof (service) - 1] = '\0';
   if ((result = getaddrinfo(addr, service, &hints, &ai)) != 0)
-    throw GAIException("unable to resolve listening address", result);
+    throw getaddrinfo_error("Unable to resolve listening address", result);
 
   try {
     createTcpListeners(listeners, ai);
@@ -463,7 +466,7 @@ void network::createTcpListeners(std::list<SocketListener*> *listeners,
 
   initSockets();
 
-  for (current = ai; current != NULL; current = current->ai_next) {
+  for (current = ai; current != nullptr; current = current->ai_next) {
     switch (current->ai_family) {
     case AF_INET:
       if (!UseIPv4)
@@ -482,7 +485,7 @@ void network::createTcpListeners(std::list<SocketListener*> *listeners,
     try {
       new_listeners.push_back(new TcpListener(current->ai_addr,
                                               current->ai_addrlen));
-    } catch (SocketException& e) {
+    } catch (socket_error& e) {
       // Ignore this if it is due to lack of address family support on
       // the interface or on the system
       if (e.err != EADDRNOTAVAIL && e.err != EAFNOSUPPORT) {
@@ -607,7 +610,7 @@ TcpFilter::Pattern TcpFilter::parsePattern(const char* p) {
 
   parts = rfb::split(&p[1], '/');
   if (parts.size() > 2)
-    throw Exception("invalid filter specified");
+    throw std::invalid_argument("Invalid filter specified");
 
   if (parts[0].empty()) {
     // Match any address
@@ -629,8 +632,8 @@ TcpFilter::Pattern TcpFilter::parsePattern(const char* p) {
         parts[0].erase(parts.size()-1, 1);
     }
 
-    if ((result = getaddrinfo (parts[0].c_str(), NULL, &hints, &ai)) != 0) {
-      throw GAIException("unable to resolve host by name", result);
+    if ((result = getaddrinfo (parts[0].c_str(), nullptr, &hints, &ai)) != 0) {
+      throw getaddrinfo_error("Unable to resolve host by name", result);
     }
 
     memcpy (&pattern.address.u.sa, ai->ai_addr, ai->ai_addrlen);
@@ -641,8 +644,8 @@ TcpFilter::Pattern TcpFilter::parsePattern(const char* p) {
     if (parts.size() > 1) {
       if (family == AF_INET &&
           (parts[1].find('.') != std::string::npos)) {
-        throw Exception("mask no longer supported for filter, "
-                        "use prefix instead");
+        throw std::invalid_argument("Mask no longer supported for "
+                                    "filter, use prefix instead");
       }
 
       pattern.prefixlen = (unsigned int) atoi(parts[1].c_str());
@@ -655,7 +658,7 @@ TcpFilter::Pattern TcpFilter::parsePattern(const char* p) {
         pattern.prefixlen = 128;
         break;
       default:
-        throw Exception("unknown address family");
+        throw std::runtime_error("Unknown address family");
       }
     }
   }
@@ -663,8 +666,9 @@ TcpFilter::Pattern TcpFilter::parsePattern(const char* p) {
   family = pattern.address.u.sa.sa_family;
 
   if (pattern.prefixlen > (family == AF_INET ? 32: 128))
-    throw Exception("invalid prefix length for filter address: %u",
-                    pattern.prefixlen);
+    throw std::invalid_argument(rfb::format("Invalid prefix length for "
+                                            "filter address: %u",
+                                            pattern.prefixlen));
 
   // Compute mask from address and prefix length
   memset (&pattern.mask, 0, sizeof (pattern.mask));
@@ -711,11 +715,11 @@ std::string TcpFilter::patternToStr(const TcpFilter::Pattern& p) {
 
   if (p.address.u.sa.sa_family == AF_INET) {
     getnameinfo(&p.address.u.sa, sizeof(p.address.u.sin),
-                addr, sizeof(addr), NULL, 0, NI_NUMERICHOST);
+                addr, sizeof(addr), nullptr, 0, NI_NUMERICHOST);
   } else if (p.address.u.sa.sa_family == AF_INET6) {
     addr[0] = '[';
     getnameinfo(&p.address.u.sa, sizeof(p.address.u.sin6),
-                addr + 1, sizeof(addr) - 2, NULL, 0, NI_NUMERICHOST);
+                addr + 1, sizeof(addr) - 2, nullptr, 0, NI_NUMERICHOST);
     strcat(addr, "]");
   } else
     addr[0] = '\0';
