@@ -29,7 +29,10 @@
 #include <stdlib.h>
 #include <stddef.h>
 
+#include <rdr/Exception.h>
+
 #include <network/UnixSocket.h>
+
 #include <rfb/LogWriter.h>
 
 using namespace network;
@@ -50,12 +53,12 @@ UnixSocket::UnixSocket(const char *path)
   socklen_t salen;
 
   if (strlen(path) >= sizeof(addr.sun_path))
-    throw SocketException("socket path is too long", ENAMETOOLONG);
+    throw socket_error("Socket path is too long", ENAMETOOLONG);
 
   // - Create a socket
   sock = socket(AF_UNIX, SOCK_STREAM, 0);
   if (sock == -1)
-    throw SocketException("unable to create socket", errno);
+    throw socket_error("Unable to create socket", errno);
 
   // - Attempt to connect
   memset(&addr, 0, sizeof(addr));
@@ -69,7 +72,7 @@ UnixSocket::UnixSocket(const char *path)
   }
 
   if (result == -1)
-    throw SocketException("unable to connect to socket", err);
+    throw socket_error("Unable to connect to socket", err);
 
   setFd(sock);
 }
@@ -84,7 +87,7 @@ const char* UnixSocket::getPeerAddress() {
 
   salen = sizeof(addr);
   if (getpeername(getFd(), (struct sockaddr *)&addr, &salen) != 0) {
-    vlog.error("unable to get peer name for socket");
+    vlog.error("Unable to get peer name for socket");
     return "";
   }
 
@@ -93,7 +96,7 @@ const char* UnixSocket::getPeerAddress() {
 
   salen = sizeof(addr);
   if (getsockname(getFd(), (struct sockaddr *)&addr, &salen) != 0) {
-    vlog.error("unable to get local name for socket");
+    vlog.error("Unable to get local name for socket");
     return "";
   }
 
@@ -116,11 +119,11 @@ UnixListener::UnixListener(const char *path, int mode)
   int err, result;
 
   if (strlen(path) >= sizeof(addr.sun_path))
-    throw SocketException("socket path is too long", ENAMETOOLONG);
+    throw socket_error("Socket path is too long", ENAMETOOLONG);
 
   // - Create a socket
   if ((fd = socket(AF_UNIX, SOCK_STREAM, 0)) < 0)
-    throw SocketException("unable to create listening socket", errno);
+    throw socket_error("Unable to create listening socket", errno);
 
   // - Delete existing socket (ignore result)
   unlink(path);
@@ -135,14 +138,14 @@ UnixListener::UnixListener(const char *path, int mode)
   umask(saved_umask);
   if (result < 0) {
     close(fd);
-    throw SocketException("unable to bind listening socket", err);
+    throw socket_error("Unable to bind listening socket", err);
   }
 
   // - Set socket mode
   if (chmod(path, mode) < 0) {
     err = errno;
     close(fd);
-    throw SocketException("unable to set socket mode", err);
+    throw socket_error("Unable to set socket mode", err);
   }
 
   listen(fd);
@@ -157,8 +160,8 @@ UnixListener::~UnixListener()
     unlink(addr.sun_path);
 }
 
-Socket* UnixListener::createSocket(int fd) {
-  return new UnixSocket(fd);
+Socket* UnixListener::createSocket(int fd_) {
+  return new UnixSocket(fd_);
 }
 
 int UnixListener::getMyPort() {
