@@ -24,6 +24,8 @@
 
 #include <rdr/TLSException.h>
 
+#include <rfb/util.h>
+
 #include <string.h>
 #include <stdio.h>
 #ifdef HAVE_GNUTLS
@@ -33,8 +35,10 @@
 using namespace rdr;
 
 #ifdef HAVE_GNUTLS
-TLSException::TLSException(const char* s, int err_)
-  : Exception("%s: %s (%d)", s, gnutls_strerror(err), err), err(err_)
+tls_error::tls_error(const char* s, int err_) noexcept
+  : std::runtime_error(rfb::format("%s: %s (%d)", s,
+                                   gnutls_strerror(err_), err_)),
+    err(err_)
 {
 }
 #endif /* HAVE_GNUTLS */
