@@ -29,10 +29,9 @@ namespace rfb {
 
   class CSecurityNone : public CSecurity {
   public:
-    CSecurityNone(CConnection* cc) : CSecurity(cc) {}
-    virtual bool processMsg() { return true; }
-    virtual int getType() const {return secTypeNone;}
-    virtual const char* description() const {return "No Encryption";}
+    CSecurityNone(CConnection* cc_) : CSecurity(cc_) {}
+    bool processMsg() override { return true; }
+    int getType() const override {return secTypeNone;}
   };
 }
 #endif
