@@ -85,11 +85,11 @@ Congestion::Congestion() :
     baseRTT(-1), congWindow(INITIAL_WINDOW), inSlowStart(true),
     safeBaseRTT(-1), measurements(0), minRTT(-1), minCongestedRTT(-1)
 {
-  gettimeofday(&lastUpdate, NULL);
-  gettimeofday(&lastSent, NULL);
+  gettimeofday(&lastUpdate, nullptr);
+  gettimeofday(&lastSent, nullptr);
   memset(&lastPong, 0, sizeof(lastPong));
-  gettimeofday(&lastPongArrival, NULL);
-  gettimeofday(&lastAdjustment, NULL);
+  gettimeofday(&lastPongArrival, nullptr);
+  gettimeofday(&lastAdjustment, nullptr);
 }
 
 Congestion::~Congestion()
@@ -101,7 +101,7 @@ void Congestion::updatePosition(unsigned pos)
   struct timeval now;
   unsigned delta, consumed;
 
-  gettimeofday(&now, NULL);
+  gettimeofday(&now, nullptr);
 
   delta = pos - lastPosition;
   if ((delta > 0) || (extraBuffer > 0))
@@ -121,7 +121,7 @@ void Congestion::updatePosition(unsigned pos)
     congWindow = __rfbmin(INITIAL_WINDOW, congWindow);
     baseRTT = -1;
     measurements = 0;
-    gettimeofday(&lastAdjustment, NULL);
+    gettimeofday(&lastAdjustment, nullptr);
     minRTT = minCongestedRTT = -1;
     inSlowStart = true;
   }
@@ -149,7 +149,7 @@ void Congestion::sentPing()
 
   memset(&rttInfo, 0, sizeof(struct RTTInfo));
 
-  gettimeofday(&rttInfo.tv, NULL);
+  gettimeofday(&rttInfo.tv, nullptr);
   rttInfo.pos = lastPosition;
   rttInfo.extra = getExtraBuffer();
   rttInfo.congested = isCongested();
@@ -166,7 +166,7 @@ void Congestion::gotPong()
   if (pings.empty())
     return;
 
-  gettimeofday(&now, NULL);
+  gettimeofday(&now, nullptr);
 
   rttInfo = pings.front();
   pings.pop_front();
@@ -320,7 +320,7 @@ void Congestion::debugTrace(const char* filename, int fd)
 #ifdef __linux__
   FILE *f;
   f = fopen(filename, "ab");
-  if (f != NULL) {
+  if (f != nullptr) {
     struct tcp_info info;
     int buffered;
     socklen_t len;
@@ -329,7 +329,7 @@ void Congestion::debugTrace(const char* filename, int fd)
                     TCP_INFO, &info, &len) == 0) &&
         (ioctl(fd, SIOCOUTQ, &buffered) == 0)) {
       struct timeval now;
-      gettimeofday(&now, NULL);
+      gettimeofday(&now, nullptr);
       fprintf(f, "%u.%06u,%u,%u,%u,%u\n",
               (unsigned)now.tv_sec, (unsigned)now.tv_usec,
               congWindow, info.tcpi_snd_cwnd * info.tcpi_snd_mss,
@@ -494,7 +494,7 @@ void Congestion::updateCongestion()
 #endif
 
   measurements = 0;
-  gettimeofday(&lastAdjustment, NULL);
+  gettimeofday(&lastAdjustment, nullptr);
   minRTT = minCongestedRTT = -1;
 }
 
