@@ -1,5 +1,5 @@
 /* Copyright (C) 2002-2005 RealVNC Ltd.  All Rights Reserved.
- * Copyright 2014-2017 Pierre Ossman for Cendio AB
+ * Copyright 2014-2023 Pierre Ossman for Cendio AB
  * 
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,27 +23,29 @@
 
 #include <assert.h>
 #include <string.h>
+
+#include <stdexcept>
+
 #include <rfb/Cursor.h>
 #include <rfb/LogWriter.h>
-#include <rfb/Exception.h>
 
 using namespace rfb;
 
 static LogWriter vlog("Cursor");
 
 Cursor::Cursor(int width, int height, const Point& hotspot,
-               const rdr::U8* data) :
+               const uint8_t* data_) :
   width_(width), height_(height), hotspot_(hotspot)
 {
-  this->data = new rdr::U8[width_*height_*4];
-  memcpy(this->data, data, width_*height_*4);
+  data = new uint8_t[width_*height_*4];
+  memcpy(data, data_, width_*height_*4);
 }
 
 Cursor::Cursor(const Cursor& other) :
   width_(other.width_), height_(other.height_),
   hotspot_(other.hotspot_)
 {
-  data = new rdr::U8[width_*height_*4];
+  data = new uint8_t[width_*height_*4];
   memcpy(data, other.data, width_*height_*4);
 }
 
@@ -81,7 +83,7 @@ static unsigned short srgb_to_lin(unsigned char srgb)
 }
 
 // Floyd-Steinberg dithering
-static void dither(int width, int height, rdr::S32* data)
+static void dither(int width, int height, int32_t* data)
 {
   for (int y = 0; y < height; y++) {
     for (int x_ = 0; x_ < width; x_++) {
@@ -124,21 +126,21 @@ static void dither(int width, int height, rdr::S32* data)
   }
 }
 
-rdr::U8* Cursor::getBitmap() const
+std::vector<uint8_t> Cursor::getBitmap() const
 {
   // First step is converting to luminance
-  rdr::S32Array luminance(width()*height());
-  rdr::S32 *lum_ptr = luminance.buf;
-  const rdr::U8 *data_ptr = data;
+  std::vector<int32_t> luminance(width()*height());
+  int32_t *lum_ptr = luminance.data();
+  const uint8_t *data_ptr = data;
   for (int y = 0; y < height(); y++) {
     for (int x = 0; x < width(); x++) {
-      rdr::S32 lum;
+      int32_t lum;
 
       // Use BT.709 coefficients for grayscale
       lum = 0;
-      lum += (rdr::U32)srgb_to_lin(data_ptr[0]) * 6947;  // 0.2126
-      lum += (rdr::U32)srgb_to_lin(data_ptr[1]) * 23436; // 0.7152
-      lum += (rdr::U32)srgb_to_lin(data_ptr[2]) * 2366;  // 0.0722
+      lum += (uint32_t)srgb_to_lin(data_ptr[0]) * 6947;  // 0.2126
+      lum += (uint32_t)srgb_to_lin(data_ptr[1]) * 23436; // 0.7152
+      lum += (uint32_t)srgb_to_lin(data_ptr[2]) * 2366;  // 0.0722
       lum /= 32768;
 
       *lum_ptr++ = lum;
@@ -147,62 +149,62 @@ rdr::U8* Cursor::getBitmap() const
   }
 
   // Then diterhing
-  dither(width(), height(), luminance.buf);
+  dither(width(), height(), luminance.data());
 
   // Then conversion to a bit mask
-  rdr::U8Array source((width()+7)/8*height());
-  memset(source.buf, 0, (width()+7)/8*height());
+  std::vector<uint8_t> source((width()+7)/8*height());
+  memset(source.data(), 0, source.size());
   int maskBytesPerRow = (width() + 7) / 8;
-  lum_ptr = luminance.buf;
+  lum_ptr = luminance.data();
   data_ptr = data;
   for (int y = 0; y < height(); y++) {
     for (int x = 0; x < width(); x++) {
       int byte = y * maskBytesPerRow + x / 8;
       int bit = 7 - x % 8;
       if (*lum_ptr > 32767)
-        source.buf[byte] |= (1 << bit);
+        source[byte] |= (1 << bit);
       lum_ptr++;
       data_ptr += 4;
     }
   }
 
-  return source.takeBuf();
+  return source;
 }
 
-rdr::U8* Cursor::getMask() const
+std::vector<uint8_t> Cursor::getMask() const
 {
   // First step is converting to integer array
-  rdr::S32Array alpha(width()*height());
-  rdr::S32 *alpha_ptr = alpha.buf;
-  const rdr::U8 *data_ptr = data;
+  std::vector<int32_t> alpha(width()*height());
+  int32_t *alpha_ptr = alpha.data();
+  const uint8_t *data_ptr = data;
   for (int y = 0; y < height(); y++) {
     for (int x = 0; x < width(); x++) {
-      *alpha_ptr++ = (rdr::U32)data_ptr[3] * 65535 / 255;
+      *alpha_ptr++ = (uint32_t)data_ptr[3] * 65535 / 255;
       data_ptr += 4;
     }
   }
 
   // Then diterhing
-  dither(width(), height(), alpha.buf);
+  dither(width(), height(), alpha.data());
 
   // Then conversion to a bit mask
-  rdr::U8Array mask((width()+7)/8*height());
-  memset(mask.buf, 0, (width()+7)/8*height());
+  std::vector<uint8_t> mask((width()+7)/8*height());
+  memset(mask.data(), 0, mask.size());
   int maskBytesPerRow = (width() + 7) / 8;
-  alpha_ptr = alpha.buf;
+  alpha_ptr = alpha.data();
   data_ptr = data;
   for (int y = 0; y < height(); y++) {
     for (int x = 0; x < width(); x++) {
       int byte = y * maskBytesPerRow + x / 8;
       int bit = 7 - x % 8;
       if (*alpha_ptr > 32767)
-        mask.buf[byte] |= (1 << bit);
+        mask[byte] |= (1 << bit);
       alpha_ptr++;
       data_ptr += 4;
     }
   }
 
-  return mask.takeBuf();
+  return mask;
 }
 
 // crop() determines the "busy" rectangle for the cursor - the minimum bounding
@@ -217,7 +219,7 @@ void Cursor::crop()
   busy = busy.intersect(Rect(hotspot_.x, hotspot_.y,
                              hotspot_.x+1, hotspot_.y+1));
   int x, y;
-  rdr::U8 *data_ptr = data;
+  uint8_t *data_ptr = data;
   for (y = 0; y < height(); y++) {
     for (x = 0; x < width(); x++) {
       if (data_ptr[3] > 0) {
@@ -234,7 +236,7 @@ void Cursor::crop()
 
   // Copy the pixel data
   int newDataLen = busy.area() * 4;
-  rdr::U8* newData = new rdr::U8[newDataLen];
+  uint8_t* newData = new uint8_t[newDataLen];
   data_ptr = newData;
   for (y = busy.tl.y; y < busy.br.y; y++) {
     memcpy(data_ptr, data + y*width()*4 + busy.tl.x*4, busy.width()*4);
@@ -253,13 +255,13 @@ RenderedCursor::RenderedCursor()
 {
 }
 
-const rdr::U8* RenderedCursor::getBuffer(const Rect& _r, int* stride) const
+const uint8_t* RenderedCursor::getBuffer(const Rect& _r, int* stride) const
 {
   Rect r;
 
   r = _r.translate(offset.negate());
   if (!r.enclosed_by(buffer.getRect()))
-    throw Exception("RenderedCursor: Invalid area requested");
+    throw std::out_of_range("RenderedCursor: Invalid area requested");
 
   return buffer.getBuffer(r, stride);
 }
@@ -270,7 +272,7 @@ void RenderedCursor::update(PixelBuffer* framebuffer,
   Point rawOffset, diff;
   Rect clippedRect;
 
-  const rdr::U8* data;
+  const uint8_t* data;
   int stride;
 
   assert(framebuffer);
@@ -300,8 +302,8 @@ void RenderedCursor::update(PixelBuffer* framebuffer,
   for (int y = 0;y < buffer.height();y++) {
     for (int x = 0;x < buffer.width();x++) {
       size_t idx;
-      rdr::U8 bg[4], fg[4];
-      rdr::U8 rgb[3];
+      uint8_t bg[4], fg[4];
+      uint8_t rgb[3];
 
       idx = (y+diff.y)*cursor->width() + (x+diff.x);
       memcpy(fg, cursor->getBuffer() + idx*4, 4);
