@@ -43,9 +43,9 @@ namespace rfb {
     JpegCompressor(int bufferLen = 128*1024);
     virtual ~JpegCompressor();
 
-    void compress(const rdr::U8 *, int, const Rect&, const PixelFormat&, int, int);
+    void compress(const uint8_t *, int, const Rect&, const PixelFormat&, int, int);
 
-    void writeBytes(const void*, int);
+    void writeBytes(const uint8_t*, int);
 
   private:
 
