@@ -34,10 +34,10 @@ namespace rfb {
   public:
     SSecurityVeNCrypt(SConnection* sc, SecurityServer *sec);
     ~SSecurityVeNCrypt();
-    virtual bool processMsg();
-    virtual int getType() const { return chosenType; }
-    virtual const char* getUserName() const;
-    virtual AccessRights getAccessRights() const;
+    bool processMsg() override;
+    int getType() const override { return chosenType; }
+    const char* getUserName() const override;
+    AccessRights getAccessRights() const override;
 
   protected:
     SSecurity *ssecurity;
