@@ -22,12 +22,14 @@
 #endif
 
 #include <assert.h>
+
+#include <stdexcept>
+
 #include <rfb/CSecurityNone.h>
 #include <rfb/CSecurityStack.h>
 #include <rfb/CSecurityVeNCrypt.h>
 #include <rfb/CSecurityVncAuth.h>
 #include <rfb/CSecurityPlain.h>
-#include <rdr/Exception.h>
 #include <rfb/Security.h>
 #ifdef HAVE_GNUTLS
 #include <rfb/CSecurityTLS.h>
@@ -38,14 +40,8 @@
 #include <rfb/CSecurityMSLogonII.h>
 #endif
 
-using namespace rdr;
 using namespace rfb;
 
-UserPasswdGetter *CSecurity::upg = NULL;
-#if defined(HAVE_GNUTLS) || defined(HAVE_NETTLE)
-UserMsgBox *CSecurity::msg = NULL;
-#endif
-
 StringParameter SecurityClient::secTypes
 ("SecurityTypes",
  "Specify which security scheme to use (None, VncAuth, Plain"
@@ -62,16 +58,10 @@ StringParameter SecurityClient::secTypes
 #ifdef HAVE_NETTLE
  "RA2,RA2_256,RA2ne,RA2ne_256,DH,MSLogonII,"
 #endif
- "VncAuth,None",
-ConfViewer);
+ "VncAuth,None");
 
 CSecurity* SecurityClient::GetCSecurity(CConnection* cc, uint32_t secType)
 {
-  assert (CSecurity::upg != NULL); /* (upg == NULL) means bug in the viewer */
-#if defined(HAVE_GNUTLS) || defined(HAVE_NETTLE)
-  assert (CSecurity::msg != NULL);
-#endif
-
   if (!IsSupported(secType))
     goto bail;
 
@@ -121,5 +111,5 @@ CSecurity* SecurityClient::GetCSecurity(CConnection* cc, uint32_t secType)
   }
 
 bail:
-  throw Exception("Security type not supported");
+  throw std::invalid_argument("Security type not supported");
 }
