@@ -33,9 +33,7 @@ namespace rfb {
     SecurityClient(void) : Security(secTypes) {}
 
     /* Create client side CSecurity class instance */
-    CSecurity* GetCSecurity(rdr::U32 secType);
-
-    static void setDefaults(void);
+    CSecurity* GetCSecurity(CConnection* cc, uint32_t secType);
 
     static StringParameter secTypes;
   };
