@@ -26,7 +26,6 @@
 #endif
 
 #include <string.h>
-#include <rfb/util.h>
 #include <rfb/ServerCore.h>
 
 rfb::IntParameter rfb::Server::idleTimeout
@@ -82,7 +81,7 @@ rfb::BoolParameter rfb::Server::acceptKeyEvents
  true);
 rfb::BoolParameter rfb::Server::acceptPointerEvents
 ("AcceptPointerEvents",
- "Accept pointer press and release events from clients.",
+ "Accept pointer movement and button events from clients.",
  true);
 rfb::BoolParameter rfb::Server::acceptCutText
 ("AcceptCutText",
