@@ -26,6 +26,8 @@
 #include <stdio.h>
 #include <sys/time.h>
 
+#include <algorithm>
+
 #include <rfb/Timer.h>
 #include <rfb/util.h>
 #include <rfb/LogWriter.h>
@@ -66,7 +68,7 @@ int Timer::checkTimeouts() {
   if (pending.empty())
     return -1;
 
-  gettimeofday(&start, 0);
+  gettimeofday(&start, nullptr);
   while (pending.front()->isBefore(start)) {
     Timer* timer;
 
@@ -84,7 +86,7 @@ int Timer::checkTimeouts() {
 
 int Timer::getNextTimeout() {
   timeval now;
-  gettimeofday(&now, 0);
+  gettimeofday(&now, nullptr);
 
   if (pending.empty())
     return -1;
@@ -97,7 +99,7 @@ int Timer::getNextTimeout() {
       return toWait;
     }
     // Time has jumped backwards!
-    vlog.info("time has moved backwards!");
+    vlog.info("Time has moved backwards!");
     pending.front()->dueTime = now;
     toWait = 0;
   }
@@ -118,7 +120,7 @@ void Timer::insertTimer(Timer* t) {
 
 void Timer::start(int timeoutMs_) {
   timeval now;
-  gettimeofday(&now, 0);
+  gettimeofday(&now, nullptr);
   stop();
   timeoutMs = timeoutMs_;
   dueTime = addMillis(now, timeoutMs);
@@ -128,7 +130,7 @@ void Timer::start(int timeoutMs_) {
 void Timer::repeat(int timeoutMs_) {
   timeval now;
 
-  gettimeofday(&now, 0);
+  gettimeofday(&now, nullptr);
 
   if (isStarted()) {
     vlog.error("Incorrectly repeating already running timer");
@@ -156,12 +158,8 @@ void Timer::stop() {
 }
 
 bool Timer::isStarted() {
-  std::list<Timer*>::iterator i;
-  for (i=pending.begin(); i!=pending.end(); i++) {
-    if (*i == this)
-      return true;
-  }
-  return false;
+  return std::find(pending.begin(), pending.end(),
+                   this) != pending.end();
 }
 
 int Timer::getTimeoutMs() {
@@ -170,7 +168,7 @@ int Timer::getTimeoutMs() {
 
 int Timer::getRemainingMs() {
   timeval now;
-  gettimeofday(&now, 0);
+  gettimeofday(&now, nullptr);
   return __rfbmax(0, diffTimeMillis(dueTime, now));
 }
 
