@@ -1,4 +1,5 @@
 /* Copyright (C) 2002-2005 RealVNC Ltd.  All Rights Reserved.
+ * Copyright (C) 2019 Brian P. Hinz
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,28 +19,25 @@
 
 package com.tigervnc.rfb;
 
+import com.tigervnc.rdr.*;
+
 public class RawDecoder extends Decoder {
 
-  public RawDecoder(CMsgReader reader_) { reader = reader_; }
+  public RawDecoder() { super(DecoderFlags.DecoderPlain); }
 
-  public void readRect(Rect r, CMsgHandler handler) {
-    int x = r.tl.x;
-    int y = r.tl.y;
-    int w = r.width();
-    int h = r.height();
-    int[] imageBuf = new int[w*h];
-    int nPixels = imageBuf.length;
-    int bytesPerRow = w * (reader.bpp() / 8);
-    while (h > 0) {
-      int nRows = nPixels / w;
-      if (nRows > h) nRows = h;
-      reader.getInStream().readPixels(imageBuf, nPixels, (reader.bpp() / 8), handler.cp.pf().bigEndian);
-      handler.imageRect(new Rect(x, y, x+w, y+nRows), imageBuf);
-      h -= nRows;
-      y += nRows;
-    }
+  public void readRect(Rect r, InStream is,
+                       ServerParams server, OutStream os)
+  {
+    os.copyBytes(is, r.area() * server.pf().bpp/8);
   }
 
-  CMsgReader reader;
   static LogWriter vlog = new LogWriter("RawDecoder");
+  public void decodeRect(Rect r, Object buffer,
+                         int buflen, ServerParams server,
+                         ModifiablePixelBuffer pb)
+  {
+    assert(buflen >= r.area() * server.pf().bpp/8);
+    pb.imageRect(server.pf(), r, (byte[])buffer);
+  }
+
 }
