@@ -1,5 +1,5 @@
 /* Copyright (C) 2002-2005 RealVNC Ltd.  All Rights Reserved.
- * Copyright (C) 2011-2016 Brian P. Hinz
+ * Copyright (C) 2011-2019 Brian P. Hinz
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -119,6 +119,7 @@ class OptionsDialog extends Dialog {
   JCheckBox encNoneCheckbox;
   JCheckBox encTLSCheckbox;
   JCheckBox encX509Checkbox;
+  JCheckBox encRSAAESCheckbox;
   JTextField caInput;
   JTextField crlInput;
   JButton caChooser;
@@ -151,9 +152,11 @@ class OptionsDialog extends Dialog {
 
   /* Misc. */
   JCheckBox sharedCheckbox;
-  JCheckBox dotWhenNoCursorCheckbox;
+  JCheckBox alwaysCursorCheckbox;
   JCheckBox acceptBellCheckbox;
 
+  JComboBox cursorTypeChoice;
+
   /* SSH */
   JCheckBox tunnelCheckbox;
   JCheckBox viaCheckbox;
@@ -174,7 +177,7 @@ class OptionsDialog extends Dialog {
   @SuppressWarnings({"rawtypes","unchecked"})
   public OptionsDialog() {
     super(true);
-    setTitle("VNC Viewer Options");
+    setTitle("VNC viewer options");
     setResizable(false);
 
     getContentPane().setLayout(
@@ -260,15 +263,6 @@ class OptionsDialog extends Dialog {
     this.dispose();
   }
 
-  public void setEmbeddedFeatures(boolean s) {
-    fullScreenCheckbox.setEnabled(s);
-    fullScreenAllMonitorsCheckbox.setEnabled(s);
-    scalingFactorInput.setEnabled(s);
-    Enumeration<AbstractButton> e = sizingGroup.getElements();
-    while (e.hasMoreElements())
-      e.nextElement().setEnabled(s);
-  }
-
   private void loadOptions()
   {
     /* Compression */
@@ -319,6 +313,7 @@ class OptionsDialog extends Dialog {
     handleAutoselect();
     handleCompression();
     handleJpeg();
+    handleAlwaysCursor();
 
     /* Security */
     Security security = new Security(SecurityClient.secTypes);
@@ -332,6 +327,7 @@ class OptionsDialog extends Dialog {
     encNoneCheckbox.setSelected(false);
     encTLSCheckbox.setSelected(false);
     encX509Checkbox.setSelected(false);
+    encRSAAESCheckbox.setSelected(false);
 
     authNoneCheckbox.setSelected(false);
     authVncCheckbox.setSelected(false);
@@ -396,6 +392,14 @@ class OptionsDialog extends Dialog {
         encX509Checkbox.setSelected(true);
         authIdentCheckbox.setSelected(true);
         break;
+      case Security.secTypeRA2:
+      case Security.secTypeRA256:
+        encRSAAESCheckbox.setSelected(true);
+      case Security.secTypeRA2ne:
+      case Security.secTypeRAne256:
+        authVncCheckbox.setSelected(true);
+        authPlainCheckbox.setSelected(true);
+        break;
       }
     }
 
@@ -457,7 +461,9 @@ class OptionsDialog extends Dialog {
 
     /* Misc. */
     sharedCheckbox.setSelected(shared.getValue());
-    dotWhenNoCursorCheckbox.setSelected(dotWhenNoCursor.getValue());
+    alwaysCursorCheckbox.setSelected(alwaysCursor.getValue());
+    String cursorTypeStr = cursorType.getValueStr();
+    cursorTypeChoice.setSelectedItem(cursorTypeStr);
     acceptBellCheckbox.setSelected(acceptBell.getValue());
 
     /* SSH */
@@ -489,7 +495,6 @@ class OptionsDialog extends Dialog {
     handleTunnel();
     handleVia();
     handleExtSSH();
-    handleEmbed();
     handleRfbState();
   }
 
@@ -526,10 +531,16 @@ class OptionsDialog extends Dialog {
     if (encNoneCheckbox.isSelected()) {
       if (authNoneCheckbox.isSelected())
         security.EnableSecType(Security.secTypeNone);
-      if (authVncCheckbox.isSelected())
+      if (authVncCheckbox.isSelected()) {
         security.EnableSecType(Security.secTypeVncAuth);
-      if (authPlainCheckbox.isSelected())
+        security.EnableSecType(Security.secTypeRA2ne);
+        security.EnableSecType(Security.secTypeRAne256);
+      }
+      if (authPlainCheckbox.isSelected()) {
         security.EnableSecType(Security.secTypePlain);
+        security.EnableSecType(Security.secTypeRA2ne);
+        security.EnableSecType(Security.secTypeRAne256);
+      }
       if (authIdentCheckbox.isSelected())
         security.EnableSecType(Security.secTypeIdent);
     }
@@ -558,6 +569,11 @@ class OptionsDialog extends Dialog {
         security.EnableSecType(Security.secTypeX509Ident);
     }
 
+    if (encRSAAESCheckbox.isSelected()) {
+      security.EnableSecType(Security.secTypeRA2);
+      security.EnableSecType(Security.secTypeRA256);
+    }
+
     if (authIdentCheckbox.isSelected() ||
         authPlainCheckbox.isSelected()) {
       sendLocalUsername.setParam(sendLocalUsernameCheckbox.isSelected());
@@ -602,8 +618,9 @@ class OptionsDialog extends Dialog {
 
     /* Misc. */
     shared.setParam(sharedCheckbox.isSelected());
-    dotWhenNoCursor.setParam(dotWhenNoCursorCheckbox.isSelected());
+    alwaysCursor.setParam(alwaysCursorCheckbox.isSelected());
     acceptBell.setParam(acceptBellCheckbox.isSelected());
+    cursorType.setParam((String)cursorTypeChoice.getSelectedItem());
 
     /* SSH */
     tunnel.setParam(tunnelCheckbox.isSelected());
@@ -678,14 +695,10 @@ class OptionsDialog extends Dialog {
 
     JPanel colorPanel = new JPanel(new GridLayout(4, 1));
     colorPanel.setBorder(BorderFactory.createTitledBorder("Color level"));
-    fullcolorButton = new GroupedJRadioButton("Full (all available colors)",
-                                              colorlevelGroup, colorPanel);
-    mediumcolorButton = new GroupedJRadioButton("Medium (256 colors)",
-                                                colorlevelGroup, colorPanel);
-    lowcolorButton = new GroupedJRadioButton("Low (64 colors)",
-                                             colorlevelGroup, colorPanel);
-    verylowcolorButton = new GroupedJRadioButton("Very low (8 colors)",
-                                                 colorlevelGroup, colorPanel);
+    fullcolorButton = new GroupedJRadioButton("Full", colorlevelGroup, colorPanel);
+    mediumcolorButton = new GroupedJRadioButton("Medium", colorlevelGroup, colorPanel);
+    lowcolorButton = new GroupedJRadioButton("Low", colorlevelGroup, colorPanel);
+    verylowcolorButton = new GroupedJRadioButton("Very low", colorlevelGroup, colorPanel);
 
     JPanel encodingPane = new JPanel(new GridLayout(1, 2, 5, 0));
     encodingPane.setBorder(BorderFactory.createEmptyBorder(0, 0, 5, 0));
@@ -705,7 +718,7 @@ class OptionsDialog extends Dialog {
     compressionInput.setPrototypeDisplayValue("0.");
     compressionInput.setEditable(true);
     JLabel compressionLabel =
-      new JLabel("Level (1=fast, 6=best [4-6 are rarely useful])");
+      new JLabel("Level (0=fast, 9=best)");
     jpegCheckbox = new JCheckBox("Allow JPEG Compression");
     jpegCheckbox.addItemListener(new ItemListener() {
       public void itemStateChanged(ItemEvent e) {
@@ -823,6 +836,12 @@ class OptionsDialog extends Dialog {
           crlInput.setText(f.getAbsolutePath());
       }
     });
+    encRSAAESCheckbox = new JCheckBox("RSA-AES");
+    encRSAAESCheckbox.addItemListener(new ItemListener() {
+      public void itemStateChanged(ItemEvent e) {
+        handleRSAAES();
+      }
+    });
     encrPanel.add(encNoneCheckbox,
                   new GridBagConstraints(0, 0,
                                          REMAINDER, 1,
@@ -887,6 +906,13 @@ class OptionsDialog extends Dialog {
                                          LINE_START, VERTICAL,
                                          new Insets(0, 5, 0, 0),
                                          0, 0));
+    encrPanel.add(encRSAAESCheckbox,
+                  new GridBagConstraints(0, 5,
+                                         REMAINDER, 1,
+                                         HEAVY, LIGHT,
+                                         LINE_START, NONE,
+                                         new Insets(0, 0, 4, 0),
+                                         NONE, NONE));
 
     JPanel authPanel = new JPanel(new GridBagLayout());
     authPanel.setBorder(BorderFactory.createTitledBorder("Authentication"));
@@ -1153,31 +1179,54 @@ class OptionsDialog extends Dialog {
     MiscPanel.setBorder(BorderFactory.createEmptyBorder(5, 5, 0, 5));
     sharedCheckbox =
       new JCheckBox("Shared (don't disconnect other viewers)");
-    dotWhenNoCursorCheckbox = new JCheckBox("Show dot when no cursor");
+    alwaysCursorCheckbox = new JCheckBox("Show local cursor when not provided by server");
+    alwaysCursorCheckbox.addItemListener(new ItemListener() {
+      public void itemStateChanged(ItemEvent e) {
+        handleAlwaysCursor();
+      }
+    });
+    JLabel cursorTypeLabel = new JLabel("Cursor type:");
+    String[] cursorTypes = {"Dot", "System"};
+    cursorTypeChoice = new MyJComboBox(cursorTypes);
+    cursorTypeChoice.setPrototypeDisplayValue("System.");
     acceptBellCheckbox = new JCheckBox("Beep when requested by the server");
     MiscPanel.add(sharedCheckbox,
                   new GridBagConstraints(0, 0,
-                                         1, 1,
+                                         REMAINDER, 1,
                                          LIGHT, LIGHT,
                                          LINE_START, NONE,
                                          new Insets(0, 0, 4, 0),
                                          NONE, NONE));
-    MiscPanel.add(dotWhenNoCursorCheckbox,
+    MiscPanel.add(alwaysCursorCheckbox,
                   new GridBagConstraints(0, 1,
-                                         1, 1,
+                                         REMAINDER, 1,
                                          LIGHT, LIGHT,
                                          LINE_START, NONE,
                                          new Insets(0, 0, 4, 0),
                                          NONE, NONE));
-    MiscPanel.add(acceptBellCheckbox,
+    MiscPanel.add(cursorTypeLabel,
                   new GridBagConstraints(0, 2,
                                          1, 1,
                                          LIGHT, LIGHT,
                                          LINE_START, NONE,
                                          new Insets(0, 0, 4, 0),
                                          NONE, NONE));
-    MiscPanel.add(Box.createRigidArea(new Dimension(5, 0)),
+    MiscPanel.add(cursorTypeChoice,
+                  new GridBagConstraints(1, 2,
+                                         1, 1,
+                                         LIGHT, LIGHT,
+                                         LINE_START, NONE,
+                                         new Insets(0, 5, 4, 0),
+                                         NONE, NONE));
+    MiscPanel.add(acceptBellCheckbox,
                   new GridBagConstraints(0, 3,
+                                         REMAINDER, 1,
+                                         LIGHT, LIGHT,
+                                         LINE_START, NONE,
+                                         new Insets(0, 0, 4, 0),
+                                         NONE, NONE));
+    MiscPanel.add(Box.createRigidArea(new Dimension(5, 0)),
+                  new GridBagConstraints(0, 4,
                                          REMAINDER, REMAINDER,
                                          HEAVY, HEAVY,
                                          LINE_START, BOTH,
@@ -1516,6 +1565,12 @@ class OptionsDialog extends Dialog {
     crlChooser.setEnabled(encX509Checkbox.isSelected());
   }
 
+  private void handleRSAAES()
+  {
+    authVncCheckbox.setSelected(true);
+    authPlainCheckbox.setSelected(true);
+  }
+
   private void handleSendLocalUsername()
   {
     boolean value = authIdentCheckbox.isSelected() ||
@@ -1589,24 +1644,10 @@ class OptionsDialog extends Dialog {
     }
   }
 
-  private void handleEmbed()
-  {
-    if (embed.getValue()) {
-      desktopSizeCheckbox.setEnabled(false);
-      desktopWidthInput.setEnabled(false);
-      desktopHeightInput.setEnabled(false);
-      remoteResizeButton.setEnabled(false);
-      remoteScaleButton.setEnabled(false);
-      fullScreenCheckbox.setEnabled(false);
-      fullScreenAllMonitorsCheckbox.setEnabled(false);
-      scalingFactorInput.setEnabled(false);
-    }
-  }
-
   private void handleRfbState()
   {
     CConn cc = VncViewer.cc;
-    if (cc != null && cc.state() == CConnection.RFBSTATE_NORMAL) {
+    if (cc != null && cc.state() == CConnection.stateEnum.RFBSTATE_NORMAL) {
       JComponent[] components = {
           encNoneCheckbox, encTLSCheckbox, encX509Checkbox, authNoneCheckbox,
           authVncCheckbox, authVncCheckbox, authIdentCheckbox, authPlainCheckbox,
@@ -1621,5 +1662,10 @@ class OptionsDialog extends Dialog {
     }
   }
 
+  private void handleAlwaysCursor()
+  {
+    cursorTypeChoice.setEnabled(alwaysCursorCheckbox.isSelected());
+  }
+
   static LogWriter vlog = new LogWriter("OptionsDialog");
 }
