@@ -37,6 +37,9 @@
 
 #include <termios.h>
 
+#ifdef HAVE_PWQUALITY
+#include <pwquality.h>
+#endif
 
 using namespace rfb;
 
@@ -44,7 +47,7 @@ char* prog;
 
 static void usage()
 {
-  fprintf(stderr,"usage: %s [file]\n", prog);
+  fprintf(stderr,"Usage: %s [file]\n", prog);
   fprintf(stderr,"       %s -f\n", prog);
   exit(1);
 }
@@ -72,7 +75,7 @@ static const char* getpassword(const char* prompt) {
       result[strlen(result)-1] = 0;
     return buf;
   }
-  return 0;
+  return nullptr;
 }
 
 // Reads passwords from stdin and prints encrypted passwords to stdout.
@@ -81,7 +84,7 @@ static int encrypt_pipe() {
 
   // We support a maximum of two passwords right now
   for (i = 0;i < 2;i++) {
-    const char *result = getpassword(NULL);
+    const char *result = getpassword(nullptr);
     if (!result)
       break;
 
@@ -99,25 +102,72 @@ static int encrypt_pipe() {
   return 0;
 }
 
+#ifdef HAVE_PWQUALITY
+static int check_passwd_pwquality(const char *password)
+{
+	int r;
+	void *auxerror;
+	pwquality_settings_t *pwq;
+	pwq = pwquality_default_settings();
+	if (!pwq)
+		return -EINVAL;
+	r = pwquality_read_config(pwq, NULL, &auxerror);
+	if (r) {
+		printf("Cannot check password quality: %s \n",
+			pwquality_strerror(NULL, 0, r, auxerror));
+		pwquality_free_settings(pwq);
+		return -EINVAL;
+	}
+
+	r = pwquality_check(pwq, password, NULL, NULL, &auxerror);
+	if (r < 0) {
+		printf("Password quality check failed:\n %s \n",
+			pwquality_strerror(NULL, 0, r, auxerror));
+		r = -EPERM;
+	}
+	pwquality_free_settings(pwq);
+
+	//return the score of password quality
+	return r;
+}
+#endif
+
 static std::vector<uint8_t> readpassword() {
   while (true) {
     const char *passwd = getpassword("Password:");
-    if (passwd == NULL) {
+    if (passwd == nullptr) {
       perror("getpassword error");
       exit(1);
     }
+
     std::string first = passwd;
-    if (first.size() < 6) {
-      if (first.empty()) {
-        fprintf(stderr,"Password not changed\n");
-        exit(1);
-      }
-      fprintf(stderr,"Password must be at least 6 characters - try again\n");
+
+    if (first.empty()) {
+      fprintf(stderr,"Password not changed\n");
+      exit(1);
+    }
+
+    if (first.size() > 8) {
+      fprintf(stderr,"Password should not be greater than 8 characters\nBecause only 8 valid characters are used - try again\n");
       continue;
     }
 
+#ifdef HAVE_PWQUALITY
+    //the function return score of password quality
+    int r = check_passwd_pwquality(passwd);
+    if (r < 0){
+      printf("Password quality check failed, please set it correctly.\n");
+      continue;
+    }
+#else
+    if (first.size() < 6) {
+      fprintf(stderr,"Password must be at least 6 characters - try again\n");
+      continue;
+    }
+#endif
+
     passwd = getpassword("Verify:");
-    if (passwd == NULL) {
+    if (passwd == nullptr) {
       perror("getpass error");
       exit(1);
     }
@@ -140,11 +190,21 @@ int main(int argc, char** argv)
   fname[0] = '\0';
 
   for (int i = 1; i < argc; i++) {
-    if (strcmp(argv[i], "-q") == 0) { // allowed for backwards compatibility
+    if ((strcmp(argv[i], "-h") == 0) ||
+        (strcmp(argv[i], "--help") == 0)) {
+      usage();
+    } else if ((strcmp(argv[i], "-v") == 0) ||
+               (strcmp(argv[i], "--version") == 0)) {
+      fprintf(stderr, "vncpasswd (TigerVNC) %s\n", PACKAGE_VERSION);
+      exit(0);
+    } else if (strcmp(argv[i], "-q") == 0) {
+      // allowed for backwards compatibility
     } else if (strncmp(argv[i], "-f", 2) == 0) {
       return encrypt_pipe();
     } else if (argv[i][0] == '-') {
-      usage();
+      fprintf(stderr, "%s: Unrecognized option '%s'\n", prog, argv[i]);
+      fprintf(stderr, "See '%s --help' for more information.\n", prog);
+      exit(1);
     } else if (fname[0] == '\0') {
       if (strlen(argv[i]) >= sizeof(fname)) {
         fprintf(stderr, "Too long filename specified\n");
@@ -152,19 +212,22 @@ int main(int argc, char** argv)
       }
       strcpy(fname, argv[i]);
     } else {
-      usage();
+      fprintf(stderr, "%s: Extra argument '%s'\n", prog, argv[i]);
+      fprintf(stderr, "See '%s --help' for more information.\n", prog);
+      exit(1);
     }
   }
 
   if (fname[0] == '\0') {
     const char *configDir = os::getvncconfigdir();
-    if (configDir == NULL) {
-      fprintf(stderr, "Can't obtain VNC config directory\n");
+    if (configDir == nullptr) {
+      fprintf(stderr, "Could not determine VNC config directory path\n");
       exit(1);
     }
     if (os::mkdir_p(configDir, 0777) == -1) {
       if (errno != EEXIST) {
-        fprintf(stderr, "Could not create VNC config directory: %s\n", strerror(errno));
+        fprintf(stderr, "Could not create VNC config directory \"%s\": %s\n",
+                configDir, strerror(errno));
         exit(1);
       }
     }
@@ -177,7 +240,7 @@ int main(int argc, char** argv)
 
     fprintf(stderr, "Would you like to enter a view-only password (y/n)? ");
     char yesno[3];
-    if (fgets(yesno, 3, stdin) != NULL && (yesno[0] == 'y' || yesno[0] == 'Y')) {
+    if (fgets(yesno, 3, stdin) != nullptr && (yesno[0] == 'y' || yesno[0] == 'Y')) {
       obfuscatedReadOnly = readpassword();
     } else {
       fprintf(stderr, "A view-only password is not used\n");
