@@ -20,6 +20,10 @@
 // PollingManager.cxx
 //
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdio.h>
 #include <string.h>
 #include <time.h>
@@ -31,6 +35,8 @@
 
 #include <x0vncserver/PollingManager.h>
 
+using namespace rfb;
+
 static LogWriter vlog("PollingMgr");
 
 const int PollingManager::m_pollingOrder[32] = {
