@@ -76,7 +76,7 @@ BoolParameter localhostOnly("localhost",
                             "Only allow connections from localhost",
                             false);
 StringParameter interface("interface",
-                          "listen on the specified network address",
+                          "Listen on the specified network address",
                           "all");
 
 static const char* defaultDesktopName()
@@ -90,10 +90,10 @@ static const char* defaultDesktopName()
     return "";
 
   struct passwd* pwent = getpwuid(getuid());
-  if (pwent == NULL)
+  if (pwent == nullptr)
     return "";
 
-  int len = snprintf(NULL, 0, "%s@%s", pwent->pw_name, hostname.data());
+  int len = snprintf(nullptr, 0, "%s@%s", pwent->pw_name, hostname.data());
   if (len < 0)
     return "";
 
@@ -154,22 +154,22 @@ class FileTcpFilter : public TcpFilter
 public:
 
   FileTcpFilter(const char *fname)
-    : TcpFilter("-"), fileName(NULL), lastModTime(0)
+    : TcpFilter("-"), fileName(nullptr), lastModTime(0)
   {
-    if (fname != NULL)
+    if (fname != nullptr)
       fileName = strdup((char *)fname);
   }
 
   virtual ~FileTcpFilter()
   {
-    if (fileName != NULL)
+    if (fileName != nullptr)
       free(fileName);
   }
 
-  virtual bool verifyConnection(Socket* s)
+  bool verifyConnection(Socket* s) override
   {
     if (!reloadRules()) {
-      vlog.error("Could not read IP filtering rules: rejecting all clients");
+      vlog.error("Could not read IP filtering rules, rejecting all clients");
       filter.clear();
       filter.push_back(parsePattern("-"));
       return false;
@@ -182,7 +182,7 @@ protected:
 
   bool reloadRules()
   {
-    if (fileName == NULL)
+    if (fileName == nullptr)
       return true;
 
     struct stat st;
@@ -192,7 +192,7 @@ protected:
     if (st.st_mtime != lastModTime) {
       // Actually reload only if the file was modified
       FILE *fp = fopen(fileName, "r");
-      if (fp == NULL)
+      if (fp == nullptr)
         return false;
 
       // Remove all the rules from the parent class
@@ -225,14 +225,14 @@ private:
 
   bool readLine(char *buf, int bufSize, FILE *fp)
   {
-    if (fp == NULL || buf == NULL || bufSize == 0)
+    if (fp == nullptr || buf == nullptr || bufSize == 0)
       return false;
 
-    if (fgets(buf, bufSize, fp) == NULL)
+    if (fgets(buf, bufSize, fp) == nullptr)
       return false;
 
     char *ptr = strchr(buf, '\n');
-    if (ptr != NULL) {
+    if (ptr != nullptr) {
       *ptr = '\0';              // remove newline at the end
     } else {
       if (!feof(fp)) {
@@ -251,7 +251,7 @@ char* programName;
 
 static void printVersion(FILE *fp)
 {
-  fprintf(fp, "TigerVNC Server version %s, built %s\n",
+  fprintf(fp, "TigerVNC server version %s, built %s\n",
           PACKAGE_VERSION, buildtime);
 }
 
@@ -279,43 +279,49 @@ int main(int argc, char** argv)
   programName = argv[0];
   Display* dpy;
 
-  Configuration::enableServerParams();
-
-  // FIXME: We don't support clipboard yet
-  Configuration::removeParam("AcceptCutText");
-  Configuration::removeParam("SendCutText");
-  Configuration::removeParam("MaxCutText");
-
   // Assume different defaults when socket activated
   if (hasSystemdListeners())
     rfbport.setParam(-1);
 
-  for (int i = 1; i < argc; i++) {
-    if (Configuration::setParam(argv[i]))
-      continue;
+  for (int i = 1; i < argc;) {
+    int ret;
 
-    if (argv[i][0] == '-') {
-      if (i+1 < argc) {
-        if (Configuration::setParam(&argv[i][1], argv[i+1])) {
-          i++;
-          continue;
-        }
-      }
-      if (strcmp(argv[i], "-v") == 0 ||
-          strcmp(argv[i], "-version") == 0 ||
-          strcmp(argv[i], "--version") == 0) {
-        printVersion(stdout);
-        return 0;
-      }
+    ret = Configuration::handleParamArg(argc, argv, i);
+    if (ret > 0) {
+      i += ret;
+      continue;
+    }
+
+    if (strcmp(argv[i], "-h") == 0 ||
+        strcmp(argv[i], "-help") == 0 ||
+        strcmp(argv[i], "--help") == 0) {
       usage();
     }
 
-    usage();
+    if (strcmp(argv[i], "-v") == 0 ||
+        strcmp(argv[i], "-version") == 0 ||
+        strcmp(argv[i], "--version") == 0) {
+      printVersion(stdout);
+      return 0;
+    }
+
+    if (argv[i][0] == '-') {
+      fprintf(stderr, "%s: Unrecognized option '%s'\n",
+              programName, argv[i]);
+      fprintf(stderr, "See '%s --help' for more information.\n",
+              programName);
+      exit(1);
+    }
+
+    fprintf(stderr, "%s: Extra argument '%s'\n", programName, argv[i]);
+    fprintf(stderr, "See '%s --help' for more information.\n",
+            programName);
+    exit(1);
   }
 
   if (!(dpy = XOpenDisplay(displayname))) {
     // FIXME: Why not vlog.error(...)?
-    fprintf(stderr,"%s: unable to open display \"%s\"\r\n",
+    fprintf(stderr,"%s: Unable to open display \"%s\"\r\n",
             programName, XDisplayName(displayname));
     exit(1);
   }
@@ -353,7 +359,7 @@ int main(int argc, char** argv)
       const char *addr = interface;
 
       if (strcasecmp(addr, "all") == 0)
-        addr = 0;
+        addr = nullptr;
       if (localhostOnly)
         createLocalTcpListeners(&tcp_listeners, (int)rfbport);
       else
@@ -368,10 +374,8 @@ int main(int argc, char** argv)
 
       FileTcpFilter fileTcpFilter(hostsFile);
       if (strlen(hostsFile) != 0)
-        for (std::list<SocketListener*>::iterator i = listeners.begin();
-             i != listeners.end();
-             i++)
-          (*i)->setFilter(&fileTcpFilter);
+        for (SocketListener* listener : listeners)
+          listener->setFilter(&fileTcpFilter);
     }
 
     if (listeners.empty()) {
@@ -395,10 +399,8 @@ int main(int argc, char** argv)
       FD_ZERO(&wfds);
 
       FD_SET(ConnectionNumber(dpy), &rfds);
-      for (std::list<SocketListener*>::iterator i = listeners.begin();
-           i != listeners.end();
-           i++)
-        FD_SET((*i)->getFd(), &rfds);
+      for (SocketListener* listener : listeners)
+        FD_SET(listener->getFd(), &rfds);
 
       server.getSockets(&sockets);
       int clients_connected = 0;
@@ -436,8 +438,8 @@ int main(int argc, char** argv)
 
       // Do the wait...
       sched.sleepStarted();
-      int n = select(FD_SETSIZE, &rfds, &wfds, 0,
-                     wait_ms >= 0 ? &tv : NULL);
+      int n = select(FD_SETSIZE, &rfds, &wfds, nullptr,
+                     wait_ms >= 0 ? &tv : nullptr);
       sched.sleepFinished();
 
       if (n < 0) {
@@ -445,16 +447,14 @@ int main(int argc, char** argv)
           vlog.debug("Interrupted select() system call");
           continue;
         } else {
-          throw rdr::SystemException("select", errno);
+          throw rdr::socket_error("select", errno);
         }
       }
 
       // Accept new VNC connections
-      for (std::list<SocketListener*>::iterator i = listeners.begin();
-           i != listeners.end();
-           i++) {
-        if (FD_ISSET((*i)->getFd(), &rfds)) {
-          Socket* sock = (*i)->accept();
+      for (SocketListener* listener : listeners) {
+        if (FD_ISSET(listener->getFd(), &rfds)) {
+          Socket* sock = listener->accept();
           if (sock) {
             server.addSocket(sock);
           } else {
@@ -486,19 +486,16 @@ int main(int argc, char** argv)
       }
     }
 
-  } catch (rdr::Exception &e) {
-    vlog.error("%s", e.str());
+  } catch (std::exception& e) {
+    vlog.error("%s", e.what());
     return 1;
   }
 
   TXWindow::handleXEvents(dpy);
 
   // Run listener destructors; remove UNIX sockets etc
-  for (std::list<SocketListener*>::iterator i = listeners.begin();
-       i != listeners.end();
-       i++) {
-    delete *i;
-  }
+  for (SocketListener* listener : listeners)
+    delete listener;
 
   vlog.info("Terminated");
   return 0;
