@@ -73,8 +73,8 @@ extern char buildtime[];
 
 #include "version-config.h"
 
-#define XVNCVERSION "TigerVNC 1.14.1"
-#define XVNCCOPYRIGHT ("Copyright (C) 1999-2024 TigerVNC Team and many others (see README.rst)\n" \
+#define XVNCVERSION "TigerVNC 1.15.0"
+#define XVNCCOPYRIGHT ("Copyright (C) 1999-2025 TigerVNC team and many others (see README.rst)\n" \
                        "See https://www.tigervnc.org for information on TigerVNC.\n")
 
 #define VNC_DEFAULT_WIDTH  1024
@@ -214,8 +214,6 @@ ddxInputThreadInit(void)
 void
 ddxUseMsg(void)
 {
-    vncPrintBanner();
-
     ErrorF("-pixdepths list-of-int support given pixmap depths\n");
     ErrorF("+/-render		   turn on/off RENDER extension support"
            "(default on)\n");
@@ -374,7 +372,7 @@ ddxProcessArgument(int argc, char *argv[], int i)
 
         if ((vncInetdSock = dup(0)) == -1)
             FatalError
-                ("Xvnc error: failed to allocate a new file descriptor for -inetd: %s\n", strerror(errno));
+                ("Xvnc error: Failed to allocate a new file descriptor for -inetd: %s\n", strerror(errno));
 
 
         /* Avoid xserver >= 1.19's epoll-fd becoming fd 2 / stderr only to be
@@ -396,7 +394,7 @@ ddxProcessArgument(int argc, char *argv[], int i)
 
                 if (displayNum == 100)
                     FatalError
-                        ("Xvnc error: no free display number for -inetd\n");
+                        ("Xvnc error: No free display number for -inetd\n");
             }
 
             display = displayNumStr;
@@ -444,7 +442,7 @@ ddxProcessArgument(int argc, char *argv[], int i)
     }
 
     if (!strcmp(argv[i], "-showconfig") || !strcmp(argv[i], "-version")) {
-        vncPrintBanner();
+        /* Already shown at start */
         exit(0);
     }
 
@@ -461,13 +459,11 @@ ddxProcessArgument(int argc, char *argv[], int i)
         }
     }
 
-    if (vncSetParamSimple(argv[i]))
-        return 1;
+    int ret;
 
-    if (argv[i][0] == '-' && i + 1 < argc) {
-        if (vncSetParam(&argv[i][1], argv[i + 1]))
-            return 2;
-    }
+    ret = vncHandleParamArg(argc, argv, i);
+    if (ret != 0)
+        return ret;
 
     return 0;
 }
@@ -1170,8 +1166,6 @@ InitOutput(ScreenInfo * scrInfo, int argc, char **argv)
     int i;
     int NumFormats = 0;
 
-    vncPrintBanner();
-
     if (serverGeneration == 1)
         LoadExtensionList(vncExtensions, ARRAY_SIZE(vncExtensions), TRUE);
 
@@ -1261,3 +1255,11 @@ vncClientGone(int fd)
         GiveUp(0);
     }
 }
+
+int
+main(int argc, char *argv[], char *envp[])
+{
+    vncPrintBanner();
+
+    return dix_main(argc, argv, envp);
+}
