@@ -61,7 +61,7 @@ std::map<OptionsCallback*, void*> OptionsDialog::callbacks;
 static std::set<OptionsDialog *> instances;
 
 OptionsDialog::OptionsDialog()
-  : Fl_Window(580, 420, _("TigerVNC Options"))
+  : Fl_Window(580, 420, _("TigerVNC options"))
 {
   int x, y;
   Fl_Navigation *navigation;
@@ -123,7 +123,7 @@ OptionsDialog::~OptionsDialog()
 
 void OptionsDialog::showDialog(void)
 {
-  static OptionsDialog *dialog = NULL;
+  static OptionsDialog *dialog = nullptr;
 
   if (!dialog)
     dialog = new OptionsDialog();
@@ -218,10 +218,8 @@ void OptionsDialog::loadOptions(void)
   Security security(SecurityClient::secTypes);
 
   list<uint8_t> secTypes;
-  list<uint8_t>::iterator iter;
 
    list<uint32_t> secTypesExt;
-   list<uint32_t>::iterator iterExt;
 
   encNoneCheckbox->value(false);
 #ifdef HAVE_GNUTLS
@@ -237,8 +235,8 @@ void OptionsDialog::loadOptions(void)
   authPlainCheckbox->value(false);
 
   secTypes = security.GetEnabledSecTypes();
-  for (iter = secTypes.begin(); iter != secTypes.end(); ++iter) {
-    switch (*iter) {
+  for (uint8_t type : secTypes) {
+    switch (type) {
     case secTypeNone:
       encNoneCheckbox->value(true);
       authNoneCheckbox->value(true);
@@ -251,8 +249,8 @@ void OptionsDialog::loadOptions(void)
   }
 
   secTypesExt = security.GetEnabledExtSecTypes();
-  for (iterExt = secTypesExt.begin(); iterExt != secTypesExt.end(); ++iterExt) {
-    switch (*iterExt) {
+  for (uint32_t type : secTypesExt) {
+    switch (type) {
     case secTypePlain:
       encNoneCheckbox->value(true);
       authPlainCheckbox->value(true);
@@ -330,9 +328,9 @@ void OptionsDialog::loadOptions(void)
   menuKeyChoice->value(0);
 
   menuKeyBuf = menuKey;
-  for (int i = 0; i < getMenuKeySymbolCount(); i++)
-    if (!strcmp(getMenuKeySymbols()[i].name, menuKeyBuf))
-      menuKeyChoice->value(i + 1);
+  for (int idx = 0; idx < getMenuKeySymbolCount(); idx++)
+    if (!strcmp(getMenuKeySymbols()[idx].name, menuKeyBuf))
+      menuKeyChoice->value(idx + 1);
 
   /* Display */
   if (!fullScreen) {
@@ -354,7 +352,14 @@ void OptionsDialog::loadOptions(void)
   /* Misc. */
   sharedCheckbox->value(shared);
   reconnectCheckbox->value(reconnectOnError);
-  dotCursorCheckbox->value(dotWhenNoCursor);
+  alwaysCursorCheckbox->value(alwaysCursor);
+  if (!strcasecmp(cursorType, "system")) {
+    cursorTypeChoice->value(1);
+  } else {
+    // Default
+    cursorTypeChoice->value(0);
+  }
+  handleAlwaysCursor(alwaysCursorCheckbox, this);
 }
 
 
@@ -488,7 +493,14 @@ void OptionsDialog::storeOptions(void)
   /* Misc. */
   shared.setParam(sharedCheckbox->value());
   reconnectOnError.setParam(reconnectCheckbox->value());
-  dotWhenNoCursor.setParam(dotCursorCheckbox->value());
+  alwaysCursor.setParam(alwaysCursorCheckbox->value());
+
+  if (cursorTypeChoice->value() == 1) {
+    cursorType.setParam("System");
+  } else {
+    // Default
+    cursorType.setParam("Dot");
+  }
 
   std::map<OptionsCallback*, void*>::const_iterator iter;
 
@@ -577,7 +589,7 @@ void OptionsDialog::createCompressionPage(int tx, int ty, int tw, int th)
 
   encodingGroup->end();
   /* Needed for resize to work sanely */
-  encodingGroup->resizable(NULL);
+  encodingGroup->resizable(nullptr);
   encodingGroup->size(encodingGroup->w(), ty - encodingGroup->y());
   col1_ty = ty;
 
@@ -629,7 +641,7 @@ void OptionsDialog::createCompressionPage(int tx, int ty, int tw, int th)
 
   colorlevelGroup->end();
   /* Needed for resize to work sanely */
-  colorlevelGroup->resizable(NULL);
+  colorlevelGroup->resizable(nullptr);
   colorlevelGroup->size(colorlevelGroup->w(),
                         ty - colorlevelGroup->y());
   col2_ty = ty;
@@ -745,7 +757,7 @@ void OptionsDialog::createSecurityPage(int tx, int ty, int tw, int th)
 
   encryptionGroup->end();
   /* Needed for resize to work sanely */
-  encryptionGroup->resizable(NULL);
+  encryptionGroup->resizable(nullptr);
   encryptionGroup->size(encryptionGroup->w(),
                         ty - encryptionGroup->y());
 
@@ -787,7 +799,7 @@ void OptionsDialog::createSecurityPage(int tx, int ty, int tw, int th)
 
   authenticationGroup->end();
   /* Needed for resize to work sanely */
-  authenticationGroup->resizable(NULL);
+  authenticationGroup->resizable(nullptr);
   authenticationGroup->size(authenticationGroup->w(),
                             ty - authenticationGroup->y());
 
@@ -796,6 +808,11 @@ void OptionsDialog::createSecurityPage(int tx, int ty, int tw, int th)
   ty += INNER_MARGIN;
 
   group->end();
+#else
+  (void)tx;
+  (void)ty;
+  (void)tw;
+  (void)th;
 #endif
 }
 
@@ -837,17 +854,29 @@ void OptionsDialog::createInputPage(int tx, int ty, int tw, int th)
                                                      _("Emulate middle mouse button")));
     ty += CHECK_HEIGHT + TIGHT_MARGIN;
 
-    dotCursorCheckbox = new Fl_Check_Button(LBLRIGHT(tx, ty,
-                                                    CHECK_MIN_WIDTH,
-                                                    CHECK_HEIGHT,
-                                                    _("Show dot when no cursor")));
+    alwaysCursorCheckbox = new Fl_Check_Button(LBLRIGHT(tx, ty,
+                                                       CHECK_MIN_WIDTH,
+                                                       CHECK_HEIGHT,
+                                                       _("Show local cursor when not provided by server")));
+    alwaysCursorCheckbox->callback(handleAlwaysCursor, this);
     ty += CHECK_HEIGHT + TIGHT_MARGIN;
+
+    /* Cursor type */
+    cursorTypeChoice = new Fl_Choice(LBLLEFT(tx, ty, 150, CHOICE_HEIGHT, _("Cursor type")));
+
+    fltk_menu_add(cursorTypeChoice, _("Dot"), 0, nullptr, nullptr, 0);
+    fltk_menu_add(cursorTypeChoice, _("System"), 0, nullptr, nullptr, 0);
+
+    fltk_adjust_choice(cursorTypeChoice);
+
+    ty += CHOICE_HEIGHT + TIGHT_MARGIN;
+
   }
   ty -= TIGHT_MARGIN;
 
   mouseGroup->end();
   /* Needed for resize to work sanely */
-  mouseGroup->resizable(NULL);
+  mouseGroup->resizable(nullptr);
   mouseGroup->size(mouseGroup->w(), ty - mouseGroup->y());
 
   /* Back to normal */
@@ -873,9 +902,11 @@ void OptionsDialog::createInputPage(int tx, int ty, int tw, int th)
 
     menuKeyChoice = new Fl_Choice(LBLLEFT(tx, ty, 150, CHOICE_HEIGHT, _("Menu key")));
 
-    fltk_menu_add(menuKeyChoice, _("None"), 0, NULL, (void*)0, FL_MENU_DIVIDER);
-    for (int i = 0; i < getMenuKeySymbolCount(); i++)
-      fltk_menu_add(menuKeyChoice, getMenuKeySymbols()[i].name, 0, NULL, 0, 0);
+    fltk_menu_add(menuKeyChoice, _("None"), 0, nullptr, nullptr, FL_MENU_DIVIDER);
+    for (int idx = 0; idx < getMenuKeySymbolCount(); idx++)
+      fltk_menu_add(menuKeyChoice, getMenuKeySymbols()[idx].name, 0, nullptr, nullptr, 0);
+
+    fltk_adjust_choice(menuKeyChoice);
 
     ty += CHOICE_HEIGHT + TIGHT_MARGIN;
   }
@@ -883,7 +914,7 @@ void OptionsDialog::createInputPage(int tx, int ty, int tw, int th)
 
   keyboardGroup->end();
   /* Needed for resize to work sanely */
-  keyboardGroup->resizable(NULL);
+  keyboardGroup->resizable(nullptr);
   keyboardGroup->size(keyboardGroup->w(), ty - keyboardGroup->y());
 
   /* Back to normal */
@@ -935,7 +966,7 @@ void OptionsDialog::createInputPage(int tx, int ty, int tw, int th)
 
   clipboardGroup->end();
   /* Needed for resize to work sanely */
-  clipboardGroup->resizable(NULL);
+  clipboardGroup->resizable(nullptr);
   clipboardGroup->size(clipboardGroup->w(), ty - clipboardGroup->y());
 
   /* Back to normal */
@@ -1013,7 +1044,7 @@ void OptionsDialog::createDisplayPage(int tx, int ty, int tw, int th)
 
   displayModeGroup->end();
   /* Needed for resize to work sanely */
-  displayModeGroup->resizable(NULL);
+  displayModeGroup->resizable(nullptr);
   displayModeGroup->size(displayModeGroup->w(),
                          ty - displayModeGroup->y());
 
@@ -1183,3 +1214,14 @@ void OptionsDialog::handleScreenConfigTimeout(void *data)
 
     self->monitorArrangement->value(fullScreenSelectedMonitors.getParam());
 }
+
+void OptionsDialog::handleAlwaysCursor(Fl_Widget* /*widget*/, void *data)
+{
+  OptionsDialog *dialog = (OptionsDialog*)data;
+
+  if (dialog->alwaysCursorCheckbox->value()) {
+    dialog->cursorTypeChoice->activate();
+  } else {
+    dialog->cursorTypeChoice->deactivate();
+  }
+}
