@@ -28,11 +28,12 @@
 #include <sys/shm.h>
 #endif
 
+#include <stdexcept>
+
 #include <FL/Fl.H>
 #include <FL/x.H>
 
 #include <rfb/LogWriter.h>
-#include <rdr/Exception.h>
 
 #include "PlatformPixelBuffer.h"
 
@@ -41,33 +42,33 @@ static rfb::LogWriter vlog("PlatformPixelBuffer");
 PlatformPixelBuffer::PlatformPixelBuffer(int width, int height) :
   FullFramePixelBuffer(rfb::PixelFormat(32, 24, false, true,
                                         255, 255, 255, 16, 8, 0),
-                       0, 0, NULL, 0),
+                       0, 0, nullptr, 0),
   Surface(width, height)
 #if !defined(WIN32) && !defined(__APPLE__)
-  , shminfo(NULL), xim(NULL)
+  , shminfo(nullptr), xim(nullptr)
 #endif
 {
 #if !defined(WIN32) && !defined(__APPLE__)
   if (!setupShm(width, height)) {
-    xim = XCreateImage(fl_display, CopyFromParent, 32,
-                       ZPixmap, 0, 0, width, height, 32, 0);
+    xim = XCreateImage(fl_display, (Visual*)CopyFromParent, 32,
+                       ZPixmap, 0, nullptr, width, height, 32, 0);
     if (!xim)
-      throw rdr::Exception("XCreateImage");
+      throw std::runtime_error("XCreateImage");
 
     xim->data = (char*)malloc(xim->bytes_per_line * xim->height);
     if (!xim->data)
-      throw rdr::Exception("malloc");
+      throw std::bad_alloc();
 
     vlog.debug("Using standard XImage");
   }
 
-  setBuffer(width, height, (rdr::U8*)xim->data,
+  setBuffer(width, height, (uint8_t*)xim->data,
             xim->bytes_per_line / (getPF().bpp/8));
 
   // On X11, the Pixmap backing this Surface is uninitialized.
   clear(0, 0, 0);
 #else
-  setBuffer(width, height, (rdr::U8*)Surface::data, width);
+  setBuffer(width, height, (uint8_t*)Surface::data, width);
 #endif
 }
 
@@ -78,15 +79,15 @@ PlatformPixelBuffer::~PlatformPixelBuffer()
     vlog.debug("Freeing shared memory XImage");
     XShmDetach(fl_display, shminfo);
     shmdt(shminfo->shmaddr);
-    shmctl(shminfo->shmid, IPC_RMID, 0);
+    shmctl(shminfo->shmid, IPC_RMID, nullptr);
     delete shminfo;
-    shminfo = NULL;
+    shminfo = nullptr;
   }
 
   // XDestroyImage() will free(xim->data) if appropriate
   if (xim)
     XDestroyImage(xim);
-  xim = NULL;
+  xim = nullptr;
 #endif
 }
 
@@ -113,7 +114,7 @@ rfb::Rect PlatformPixelBuffer::getDamage(void)
 
   GC gc;
 
-  gc = XCreateGC(fl_display, pixmap, 0, NULL);
+  gc = XCreateGC(fl_display, pixmap, 0, nullptr);
   if (shminfo) {
     XShmPutImage(fl_display, pixmap, gc, xim,
                  r.tl.x, r.tl.y, r.tl.x, r.tl.y,
@@ -135,7 +136,8 @@ rfb::Rect PlatformPixelBuffer::getDamage(void)
 
 static bool caughtError;
 
-static int XShmAttachErrorHandler(Display *dpy, XErrorEvent *error)
+static int XShmAttachErrorHandler(Display* /*dpy*/,
+                                  XErrorEvent* /*error*/)
 {
   caughtError = true;
   return 0;
@@ -146,7 +148,7 @@ bool PlatformPixelBuffer::setupShm(int width, int height)
   int major, minor;
   Bool pixmaps;
   XErrorHandler old_handler;
-  const char *display_name = XDisplayName (NULL);
+  const char *display_name = XDisplayName(nullptr);
 
   /* Don't use MIT-SHM on remote displays */
   if (*display_name && *display_name != ':')
@@ -157,8 +159,8 @@ bool PlatformPixelBuffer::setupShm(int width, int height)
 
   shminfo = new XShmSegmentInfo;
 
-  xim = XShmCreateImage(fl_display, CopyFromParent, 32,
-                        ZPixmap, 0, shminfo, width, height);
+  xim = XShmCreateImage(fl_display, (Visual*)CopyFromParent, 32,
+                        ZPixmap, nullptr, shminfo, width, height);
   if (!xim)
     goto free_shminfo;
 
@@ -168,8 +170,8 @@ bool PlatformPixelBuffer::setupShm(int width, int height)
   if (shminfo->shmid == -1)
     goto free_xim;
 
-  shminfo->shmaddr = xim->data = (char*)shmat(shminfo->shmid, 0, 0);
-  shmctl(shminfo->shmid, IPC_RMID, 0); // to avoid memory leakage
+  shminfo->shmaddr = xim->data = (char*)shmat(shminfo->shmid, nullptr, 0);
+  shmctl(shminfo->shmid, IPC_RMID, nullptr); // to avoid memory leakage
   if (shminfo->shmaddr == (char *)-1)
     goto free_xim;
 
@@ -201,11 +203,11 @@ free_shmaddr:
 
 free_xim:
   XDestroyImage(xim);
-  xim = NULL;
+  xim = nullptr;
 
 free_shminfo:
   delete shminfo;
-  shminfo = NULL;
+  shminfo = nullptr;
 
   return 0;
 }
